(* ::Package:: *)

BeginPackage["Bulatov`sdp1`"];

solveDualSDP::usage="solveDualSDP[H,B,c] finds PSD nxn matrix Z maximizing H.Z with element constraints. Nonzero entries of nxn B indicate entries to be constrained, c is n-vector of their values";
symsqrt::usage="symsqrt[m] gives symmetric square root of matrix m";
gramRound::usage="gramRound[gram] finds vectors with given gram matrix, takes random hyperplane and returns list of 1,-1 indicating which side of hyperplane each vector falls on";

Begin["`Private`"]

SetAttributes[toPySym,{Listable,HoldAll}];
SetAttributes[topy,HoldAll];
toPySym[sym_]:=(xx=SymbolName@Unevaluated[sym];Global`ToPy[xx,sym];)
topy[syms__]:=toPySym[{syms}];

solveDualSDP[Hp_,B_,cp_]:=(
	H=-Hp//Normal//N;
	n=Length@H;
	mg[pos_]:=Array[Boole[Sort[{#1,#2}]===pos]&,{n,n}];
	poses=Position[Unitize@Normal@B,1];
	G=Flatten[mg[#]]&/@poses//N;
	c=-cp//N;
	n=Length[B];
	topy[c,G,H];

	Assert[ValueQ[Global`Py["1"]]];
	Global`Py["from cvxopt import matrix, solvers"];
	Global`Py["sol=solvers.sdp(matrix(c),Gs=[matrix(G)],hs=[matrix(H)]);"];
	Partition[Global`Py["list(sol['zs'][0])"],n]
);


symsqrt[m_]:=#[[1]].Sqrt@#[[2]].Transpose@#[[1]]&@SingularValueDecomposition@m;

gramRound[gram_]:=(
	n=Length@gram;
	v=Normalize@RandomReal[{-1,1},n];
	If[#.v>0,1,-1]&/@symsqrt@gram
);
End[];
EndPackage[];
