(* ::Package:: *)

BeginPackage["Bulatov`sdp2`"];

solveDualSDP::usage="solveDualSDP[H,B,c] finds PSD nxn matrix Z maximizing H.Z with element constraints. Nonzero entries of nxn B indicate entries to be constrained, c is n-vector of their values";
symsqrt::usage="symsqrt[m] gives symmetric square root of matrix m";
gramRound::usage="gramRound[gram] finds vectors with given gram matrix, takes random hyperplane and returns list of 1,-1 indicating which side of hyperplane each vector falls on";

minimizeSpectralRadius::usage="minimizeSpectralRadius[B,V] gets symmetric 0-diagonal matrix with given element constraints and smallest spectral radius"
lovaszThetaCertificate::usage="lovaszThetaCertificate[graph] gets smallest spectral radius symmetric matrix that has same entries as graph's adjacency matrix"
lovaszTheta::usage="lovaszTheta[g] finds value of Lovasz Theta function on graph g"

Begin["`Private`"]

SetAttributes[toPySym,{Listable,HoldAll}];
SetAttributes[topy,HoldAll];
toPySym[sym_]:=(xx=SymbolName@Unevaluated[sym];Global`ToPy[xx,sym];)
topy[syms__]:=toPySym[{syms}];

solveDualSDP[Hp_,B_,cp_]:=(
	H=-Hp//Normal//N;
	n=Length@H;
	mg[pos_]:=Array[Boole[Sort[{#1,#2}]===pos]&,{n,n}];
	poses=Position[Unitize@Normal@B,1];
	G=Flatten[mg[#]]&/@poses//N;
	c=-cp//N;
	n=Length[B];
	topy[c,G,H];

	Assert[ValueQ[Global`Py["1"]]];
	Global`Py["from cvxopt import matrix, solvers"];
	Global`Py["sol=solvers.sdp(matrix(c),Gs=[matrix(G)],hs=[matrix(H)]);"];
	Partition[Global`Py["list(sol['zs'][0])"],n]
);


symsqrt[m_]:=#[[1]].Sqrt@#[[2]].Transpose@#[[1]]&@SingularValueDecomposition@m;

gramRound[gram_]:=(
	n=Length@gram;
	v=Normalize@RandomReal[{-1,1},n];
	If[#.v>0,1,-1]&/@symsqrt@gram
);

(* get symmetric 0-diagonal matrix with given element constraints and smallest spectral radius *)
minimizeSpectralRadius[B_List,cpmat_]:=(
	Assert[B==Transpose[B]];
	Assert[Union[Flatten[B]]\[Backslash]{0,1}=={}];

	n=Length@B;
	(* matrix representing difference of elements *)
	equalMat[pos1_,pos2_]:=Array[(p=Sort[{#1,#2}];Which[p==pos1,1,p==pos2,-1,True,0])&,{n,n}];
	(* matrix representing single element *)
	mg[pos_]:=Array[Boole[Sort[{#1,#2}]===pos]&,{n,n}];

	n=Length[B];
	poses=Select[Position[B,1],OrderedQ];
	cpvals=Extract[cpmat,poses];

	diagMats=equalMat[{1,1},{#,#}]&/@Range[2,n];
	elemMats=mg/@poses;
	G=(Flatten/@diagMats)~Join~(Flatten/@elemMats)//N;
	c=ConstantArray[0,Length@diagMats]~Join~(2cpvals)//N;
	H=mg[{1,1}]//N;
	topy[c,G,H];

	Global`Py["from cvxopt import matrix, solvers"];Global`Py["sol=solvers.sdp(matrix(c),Gs=[matrix(G)],hs=[matrix(H)]);"];
	Z=Partition[Global`Py["list(sol['zs'][0])"],n];
	Z2=Array[If[#1==#2,0,-Z[[#1,#2]]]&,{n,n}]
);

lovaszThetaCertificate[g_Graph]:=(
	mat=Normal@AdjacencyMatrix@g;
	minimizeSpectralRadius[mat,mat]
);

lovaszTheta[g_Graph]:=(
	cert=lovaszThetaCertificate@GraphComplement@g;
	Max@Eigenvalues@cert+1
);


End[];
EndPackage[];
