(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[      6218,        184]
NotebookOptionsPosition[      5962,        170]
NotebookOutlinePosition[      6318,        186]
CellTagsIndexPosition[      6275,        183]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"On", "[", "Assert", "]"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`sdp2`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Needs", "[", "\"\<Bulatov`common`\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"theta", " ", "of", " ", "vertex"}], "-", 
    RowBox[{
    "transitive", " ", "graph", " ", "and", " ", "its", " ", "complement", 
     " ", "should", " ", "give", " ", "number", " ", "of", " ", 
     "vertices"}]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"vertexTransitiveCheck", "[", "g_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"t1", "=", 
       RowBox[{"lovaszTheta", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"t2", "=", 
       RowBox[{"lovaszTheta", "[", 
        RowBox[{"GraphComplement", "@", "g"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"t1", " ", "t2"}], "-", 
         RowBox[{"Length", "@", 
          RowBox[{"VertexList", "@", "g"}]}]}], "<", ".001"}], "]"}], ";"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"theta", " ", "of", " ", 
    RowBox[{"graph", "'"}], "s", " ", "complement", " ", "is", " ", "equal", 
    " ", "to", " ", "clique", " ", "and", " ", "chromatic", " ", "number"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"perfectionCheck", "[", "gname_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"t1", "=", 
       RowBox[{"lovaszTheta", "[", 
        RowBox[{"GraphComplement", "[", 
         RowBox[{"GraphData", "[", "gname", "]"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"t2", "=", 
       RowBox[{"GraphData", "[", 
        RowBox[{"gname", ",", "\"\<ChromaticNumber\>\""}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"t1", "-", "t2"}], "<", ".001"}], "]"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "check", " ", "our", " ", "implementation", " ", "against", " ", "known", 
    " ", "formula", " ", "for", " ", "cycles"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cycleCheck", "[", "n_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"t1", "=", 
       RowBox[{"lovaszTheta", "[", 
        RowBox[{"CycleGraph", "[", "n", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"t2", "=", 
       FractionBox[
        RowBox[{"n", " ", 
         RowBox[{"Cos", "[", 
          FractionBox["\[Pi]", "n"], "]"}]}], 
        RowBox[{"1", "+", 
         RowBox[{"Cos", "[", 
          FractionBox["\[Pi]", "n"], "]"}]}]]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"t1", "-", "t2"}], "<", ".001"}], "]"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"test", " ", "on", " ", "some", " ", "non"}], "-", 
    RowBox[{"perfect", " ", "vertex", " ", "transitive", " ", "graphs"}]}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"vertexTransitiveCheck", "[", 
       RowBox[{"GraphData", "[", "#", "]"}], "]"}], ";"}], "&"}], "/@", 
    RowBox[{"{", 
     RowBox[{"\"\<PetersenGraph\>\"", ",", 
      RowBox[{"{", 
       RowBox[{"\"\<Antiprism\>\"", ",", "4"}], "}"}], ",", 
      "\"\<IcosahedralGraph\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "test", " ", "perfection", " ", "property", " ", "on", " ", "some", " ", 
     "planar", " ", "perfect", " ", "non"}], "-", 
    RowBox[{"regular", " ", "graphs"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"perfectionCheck", "[", "#", "]"}], ";"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<Grid\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2", ",", "3"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<Book\>\"", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<KnightsTour\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cycleCheck", "[", "#", "]"}], ";"}], "&"}], "/@", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "7", ",", "9"}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.508212549647739*^9, 3.508212647531302*^9}, {
  3.508213718433549*^9, 3.508213740350979*^9}, {3.508213790580719*^9, 
  3.5082138025856256`*^9}, {3.5082139079159813`*^9, 3.508213910553323*^9}, {
  3.508213971006069*^9, 3.508214002252583*^9}}]
},
WindowSize->{740, 752},
WindowMargins->{{160, Automatic}, {3, Automatic}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 5401, 148, 686, "Input"]
}
]
*)

(* End of internal cache information *)
